#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <curses.h>
#include <signal.h>
#include <sys/time.h>
#include <unistd.h>

#include "config.h"
#include "misc.h"
#include "screen.h"

// beep on crash
#define	beep()	write(1,"\007",1);

// tick speed
int speed = 100000;

// Game mode
char *Mode;

// options
static void get_options(int c, char *op[])
{
	register int i;
	if (c > 1)
	{
		for (i=1; i<c; i++)
		{
			switch(op[i][0])
			{
			default:
			case '?':
				(void) fprintf(stderr, "Usage: [-v] version | [-h] help | [-e] easy (normal) | [-f] fast | [-i] insane\n");
				exit(0);
				break;
			case '-':
			switch(op[i][1])
				{
				// Start in easy mode
				case 'e':
					speed = 100000;
					Mode = "Easy";
					break;
				// fast mode
				case 'f':
					speed = 60000;
					Mode = "Fast";
					break;
				// insane mode
				case 'i':
					speed = 30000;
					Mode = "Insane";
					break;
				case 'v':
				(void) fprintf(stderr, "Version 1.4 (c) 2007 Luke Th. Bullock <luke@alge.anart.no>\nVersion 1.3 (c) 2005 Luke Th. Bullock <luke@alge.anart.no>\nNibbles 1.2 (c) 2002 Nils Magnus Englund <nmee@nmee.net>\n");
					exit(0);
					break;
				case 'h':
				(void) fprintf(stderr, "usage: [-v] version | [-h] help | [-e] easy (normal) | [-f] fast | [-i] insane\n");
					exit(0);
					break;
				}
			}
		}
	}
}


// exit function
int quit (int sig)
{
	endgraphics();
	exit(0);
}

int main (int argc, char *argv[])
{

	// The tree windows (Main,Grid,Status in that order) used by curses
	WINDOW		*w [3];


	// The grid, to make it easy to tell where the worm is
	bool		grid [GRIDX] [GRIDY];

	// The worm history
	short int	hx [HISTORYSIZE];
	short int	hy [HISTORYSIZE];

	// Worm variables
	int	length = STARTLENGTH; // the length of the worm
	int	extralength = STARTELENGTH; // how much the worm is going to grow
	int	curx,cury; // current position of the worm
	int	dir = RIGHT; // direction the worm is facing
	int	lastdir = RIGHT; // last direction
	int	score = 0;
	bool	paused = false;
	int	dotx,doty; // position of the dot (worm food)
	int	tick; // current tick

	// User input
	int	ch;

	// get options
	Mode = "Easy";
	get_options(argc, argv);

	// Initialize curses and the windows
	startgraphics(w);

	// Seed the random number generator
	randomize();

	// clear the grid
	memset(grid,0,sizeof(grid));

	// Initialize and draw the worm
	curx = (GRIDX+length) >> 1;
	cury = GRIDY >> 1;
	for (tick=0;tick<length;tick++) {
		hx[tick] = curx - (length - tick) + 1;
		hy[tick] = cury;
		grid[curx-(length-tick)+1][cury] = true;
		drawblock(w,curx-(length-tick)+1,cury);
	}
	
	// Initialize and draw the dot (worm food)
	newdot(w,grid,&dotx,&doty);
	
	// Main loop
	while (1) {
		lastdir = dir;
		
		if (paused) {
			ch = wgetch(w[Grid]);
			while (ch != ERR) {
				switch (ch) {
					case 'p':
						paused = false;
						break;
					case 'q':
						endgraphics();
						beep();
						printf("Game over: You quit.\nScore: Your score was %d.\nLength: Your wormlength was %d.\nMode: Game mode was %s.\nThank you for playing Nibbles.\n",score, length, Mode);
						exit(0);
						break;
				}
				ch = wgetch(w[Grid]);
			}
			mvwprintw(w[Status],0,0,"[PAUSED] Score: %d | Length: %d | Mode: %s ",score, length, Mode);
			wrefresh(w[Status]);
			usleep(speed);
			continue;
		}

		// catch user input
		ch = wgetch(w[Grid]);
		while (ch != ERR) {
			switch (ch) {
				case KEY_LEFT:
				case 'h': // accept vi-style directions as well
					if ((lastdir == UP) || (lastdir == DOWN)) {
						dir = LEFT;
					}
					break;
				case KEY_RIGHT:
				case 'l':
					if ((lastdir == UP) || (lastdir == DOWN)) {
						dir = RIGHT;
					}
					break;
				case KEY_UP:
				case 'k':
					if ((lastdir == LEFT) || (lastdir == RIGHT)) {
						dir = UP;
					}
					break;
				case KEY_DOWN:
				case 'j':
					if ((lastdir == LEFT) || (lastdir == RIGHT)) {
						dir = DOWN;
					}
					break;
				case 'p':
					paused = true;
					break;
				case 'q':
					endgraphics();
					beep();
					printf("Game over: You quit.\nYour score was %d.\nLength: Wormlength was %d.\nMode: Game mode was %s.\nThank you for playing Nibbles.\n",score, length, Mode);
					exit(0);
					break;
			}
			ch = wgetch(w[Grid]);
		}

		// modify the new position based on the direction
		switch (dir) {
			case LEFT:	curx--; break;
			case RIGHT:	curx++; break;
			case UP:	cury--; break;
			case DOWN:	cury++; break;
		}

		// check whether or not the worm has crashed in a wall
		if ((curx<0) || (curx>=GRIDX) || (cury<0) || (cury>=GRIDY)) {
			endgraphics();
			beep();
		// lukeb
			printf("Game over: You crashed into the wall.\nScore: Your score was %d.\nLength: Worm length was %d.\nMode: Game mode was %s.\nThank you for playing Nibbles.\n", score, length, Mode);
			exit(0);
		}

		// check whether or not the worm has crashed with itself
		if (grid[curx][cury]) {
			endgraphics();
			beep();
			printf("Game over: You crashed with yourself.\nScore: Your score was %d.\nLength: Wormlength was %d.\nMode: Game mode was %s.\nThank you for playing Nibbles.\n", score, length, Mode);
			exit(0);
		}
		
		// the worm is still on the grid, and it seems healty :)

		drawblock(w,curx,cury); // let's draw the new block
		grid[curx][cury] = true; // and update the grid
		
		// update the history array
		hx[tick] = curx;
		hy[tick] = cury;
		
		// has the worm eaten food?
		if ((dotx == curx) && (doty == cury)) {
			score++;
			extralength += WORMGROWTH;
			newdot(w,grid,&dotx,&doty);
		}

		// if the worm still has some growing to do..
		if (extralength > 0) {
			extralength--;
			length++;
		} else {
			// otherwise, clear the last block
			clearblock(w,hx[tick-length],hy[tick-length]);
			grid[hx[tick-length]][hy[tick-length]] = 0;
		}

		wrefresh(w[Grid]);

		// finally, update the status line
		mvwprintw(w[Status],0,0,"[NIBBLES] Score: %d | Length: %d | Mode: %s (Type -h for help)",score, length, Mode);
		mvwprintw(w[Status],0,GRIDX - 2 - 10 ,"");
		wrefresh(w[Status]);
		
		usleep(speed);
		tick++;

		// if the history array is full, copy the length of the worm
		// to the beginning of the array and decrease the tick counter
		if (tick == HISTORYSIZE) {
			memcpy(hx, hx+HISTORYSIZE-length, length * sizeof(short int));
			memcpy(hy, hy+HISTORYSIZE-length, length * sizeof(short int));
			tick = length;
		}

		// ready for next tick!
	}
	endgraphics();
}
